@@ -161,13 +161,49 @@ mod tests {
         }
 
         let mut pq = PriorityQueue::new();
-        pq.push(MyFn { name: "increment-one", func: |x| *x += 1 }, 2);
-        pq.push(MyFn { name: "increment-two", func: |x| *x += 2 }, 1);
+        pq.push(
+            MyFn {
+                name: "increment-one",
+                func: |x| *x += 1,
+            },
+            2,
+        );
+        pq.push(
+            MyFn {
+                name: "increment-two",
+                func: |x| *x += 2,
+            },
+            1,
+        );
 
         let mut cnt = 0;
-        assert_eq![pq.peek(), Some((&MyFn { name: "increment-one", func: |_| {} }, &2))];
-        pq.change_priority(&MyFn { name: "increment-one", func: |_| {} }, 0);
-        assert_eq![pq.peek(), Some((&MyFn { name: "increment-two", func: |_| {} }, &1))];
+        assert_eq![
+            pq.peek(),
+            Some((
+                &MyFn {
+                    name: "increment-one",
+                    func: |_| {}
+                },
+                &2
+            ))
+        ];
+        pq.change_priority(
+            &MyFn {
+                name: "increment-one",
+                func: |_| {},
+            },
+            0,
+        );
+        assert_eq![
+            pq.peek(),
+            Some((
+                &MyFn {
+                    name: "increment-two",
+                    func: |_| {}
+                },
+                &1
+            ))
+        ];
 
         assert_eq![cnt, 0];
 
@@ -254,10 +290,7 @@ mod tests {
         let v = vec![("c", 4), ("d", 6), ("e", 3)];
         pq.extend(v);
         assert_eq!(pq.len(), 3);
-        assert_eq!(
-            pq.into_sorted_vec().as_slice(),
-            &["d", "c", "e"]
-        );
+        assert_eq!(pq.into_sorted_vec().as_slice(), &["d", "c", "e"]);
     }
 
     #[test]
@@ -340,71 +373,12 @@ mod tests {
         type PqType = PriorityQueue<i32, Instant>;
         let _: PqType = PriorityQueue::default();
     }
-
-    #[cfg(feature = "benchmarks")]
-    extern crate test;
-    #[cfg(feature = "benchmarks")]
-    use test::{black_box, Bencher};
-
-    #[cfg(feature = "benchmarks")]
-    #[bench]
-    fn push_and_pop(b: &mut Bencher) {
-        type PqType = PriorityQueue<usize, i32>;
-        let mut pq: PqType = PriorityQueue::new();
-        b.iter(|| {
-            pq.push(black_box(0), black_box(0));
-            assert_eq![pq.pop().unwrap().1, 0];
-        });
-    }
-
-    //for comparison, using the BinaryHeap
-    #[cfg(feature = "benchmarks")]
-    #[bench]
-    fn push_and_pop_std(b: &mut Bencher) {
-        use std::collections::BinaryHeap;
-        type PqType = BinaryHeap<(usize, i32)>;
-        let mut pq: PqType = BinaryHeap::new();
-        b.iter(|| {
-            pq.push((black_box(0), black_box(0)));
-            assert_eq![pq.pop().unwrap().1, 0];
-        });
-    }
-
-    #[cfg(feature = "benchmarks")]
-    #[bench]
-    fn push_and_pop_on_large_queue(b: &mut Bencher) {
-        type PqType = PriorityQueue<usize, i32>;
-        let mut pq: PqType = PriorityQueue::new();
-        for i in 0..100_000 {
-            pq.push(black_box(i as usize), black_box(i));
-        }
-        b.iter(|| {
-            pq.push(black_box(100_000), black_box(100_000));
-            assert_eq![pq.pop().unwrap().1, black_box(100_000)];
-        });
-    }
-
-    #[cfg(feature = "benchmarks")]
-    #[bench]
-    fn push_and_pop_on_large_queue_std(b: &mut Bencher) {
-        use std::collections::BinaryHeap;
-        type PqType = BinaryHeap<(usize, i32)>;
-        let mut pq: PqType = BinaryHeap::new();
-        for i in 0..100_000 {
-            pq.push((black_box(i as usize), black_box(i)));
-        }
-        b.iter(|| {
-            pq.push((black_box(100_000), black_box(100_000)));
-            assert_eq![pq.pop().unwrap().1, black_box(100_000)];
-        });
-    }
-
 }
 
 #[cfg(all(feature = "serde", test))]
 mod serde_tests_basics {
-    use serde_test::{assert_tokens, Token};
     use crate::PriorityQueue;
+    use serde_test::{assert_tokens, Token};
     #[test]
     fn serde_empty() {
         let pq: PriorityQueue<String, i32> = PriorityQueue::new();
@@ -456,11 +430,11 @@ mod serde_tests_basics {
 //thanks to ckaran
 #[cfg(all(feature = "serde", test))]
 mod serde_tests_custom_structs {
-    use uuid::Uuid;
     use crate::PriorityQueue;
     use std::cmp::{Ord, Ordering, PartialOrd};
     use std::default::Default;
     use std::time::Duration;
+    use uuid::Uuid;
 
     // Abusing Duration as a mutable std::time::Instant
     type ActivationDate = Duration;
@@ -644,3 +618,81 @@ mod serde_tests_custom_structs {
         }
     }
 }
+
+#[cfg(all(test, feature = "benchmarks"))]
+mod benchmarks {
+    extern crate test;
+    use crate::PriorityQueue;
+    use test::{black_box, Bencher};
+    use hashbrown::hash_map::DefaultHashBuilder;
+    
+    #[bench]
+    fn push_and_pop(b: &mut Bencher) {
+        type PqType = PriorityQueue<usize, i32>;
+        let mut pq: PqType = PriorityQueue::new();
+        b.iter(|| {
+            pq.push(black_box(0), black_box(0));
+            assert_eq![pq.pop().unwrap().1, 0];
+        });
+    }
+
+    #[bench]
+    fn push_and_pop_fx(b: &mut Bencher) {
+        let mut pq = PriorityQueue::<_, _, DefaultHashBuilder>::with_default_hasher();
+        b.iter(|| {
+            pq.push(black_box(0), black_box(0));
+            assert_eq![pq.pop().unwrap().1, 0];
+        });
+    }
+
+    //for comparison, using the BinaryHeap
+    #[bench]
+    fn push_and_pop_std(b: &mut Bencher) {
+        use std::collections::BinaryHeap;
+        type PqType = BinaryHeap<(usize, i32)>;
+        let mut pq: PqType = BinaryHeap::new();
+        b.iter(|| {
+            pq.push((black_box(0), black_box(0)));
+            assert_eq![pq.pop().unwrap().1, 0];
+        });
+    }
+
+    #[bench]
+    fn push_and_pop_on_large_queue(b: &mut Bencher) {
+        type PqType = PriorityQueue<usize, i32>;
+        let mut pq: PqType = PriorityQueue::new();
+        for i in 0..100_000 {
+            pq.push(black_box(i as usize), black_box(i));
+        }
+        b.iter(|| {
+            pq.push(black_box(100_000), black_box(100_000));
+            assert_eq![pq.pop().unwrap().1, black_box(100_000)];
+        });
+    }
+    #[bench]
+    fn push_and_pop_on_large_queue_fx(b: &mut Bencher) {
+        let mut pq = PriorityQueue::<_, _, DefaultHashBuilder>::with_default_hasher();
+        for i in 0..100_000 {
+            pq.push(black_box(i as usize), black_box(i));
+        }
+        b.iter(|| {
+            pq.push(black_box(100_000), black_box(100_000));
+            assert_eq![pq.pop().unwrap().1, black_box(100_000)];
+        });
+    }
+
+    #[bench]
+    fn push_and_pop_on_large_queue_std(b: &mut Bencher) {
+        use std::collections::BinaryHeap;
+        type PqType = BinaryHeap<(usize, i32)>;
+        let mut pq: PqType = BinaryHeap::new();
+        for i in 0..100_000 {
+            pq.push((black_box(i as usize), black_box(i)));
+        }
+        b.iter(|| {
+            pq.push((black_box(100_000), black_box(100_000)));
+            assert_eq![pq.pop().unwrap().1, black_box(100_000)];
+        });
+    }
+
+}
